unit drives; { drive access, ab MS-DOS 4.0 }

{ written by Thorsten Reinecke }


interface

function SectorSize (Disk : char) : word;
function totalDataSectors (Disk : char) : longint;
function totalSectors (Disk : char) : longint;
function firstDataSector (Disk : char) : word;
function MegaBytes (Disk : char) : word;
function SectorRead (Drive : char; first : longint; anz : word; var buffer) : boolean;
function SectorWrite (Drive : char; first : longint; anz : word; var buffer) : boolean;


{ Hinweis:
 Sector Read/Write liefert true zurck, wenn erfolgreich; false bei Fehler
 Sector Read/Write ist nullbezogen,d.h.
  - erster Sektor ist 0,
  - letzter Sektor ist totalSectors(Drive)-1   }

implementation

function SectorSize (Disk : char) : word; assembler;
asm
 mov dl,Disk; or dl,$20; { upcase }
 sub dl,'a'-1; { und Laufwerksnummer ermitteln; hier 0: default; 1:A 2:B usw. }
 mov ah,$36; int $21; { DOS-call }
 inc ax; je @ende; { in diesem Fall ungltige Laufwerksbezeichnung }
 mov ax,cx;
@ende:
end;

function totalDataSectors (Disk : char) : longint; assembler;
asm
 mov dl,Disk; or dl,$20; { upcase }
 sub dl,'a'-1; { und Laufwerksnummer ermitteln }
 mov ah,$36; int $21; { DOS-call }
 inc ax; jz @weiter; { in diesem Fall ungltige Laufwerksbezeichnung }
 dec ax;
@weiter: mul dx; { #Sektoren pro Cluster * #Cluster }
@ende:
end;

function MegaBytes (Disk : char) : word; assembler;
asm
 mov dl,Disk; or dl,$20; { upcase }
 sub dl,'a'-1; { und Laufwerksnummer ermitteln }
 mov ah,$36; int $21; { DOS-call }
 inc ax; jz @ende; { in diesem Fall ungltige Laufwerksbezeichnung }
 dec ax;
@weiter: mul dx; { #Sektoren pro Cluster * #Cluster }
 push ax; push dx;
 mov dx,$10; mov ax,0; div cx; mov cx,ax; { #Cluster pro Megabyte }
 pop dx; pop ax; div cx; { #Megabytes insgesamt }
 shr cx,1; cmp dx,cx; jb @ende; inc ax; { ggf. aufrunden }
@ende:
end;


function firstDataSector (Disk : char) : word; assembler;
asm
 push ds;
 mov dl,Disk; or dl,$20; { upcase }
 sub dl,'a'-1; { und Laufwerksnummer ermitteln }
 mov ah,$32; int $21; { DOS-call }
 cmp al,00; je @ok;
 { Fehler: Tabelle existiert gar nicht }
 xor ax,ax; jmp @ende;
@ok:
 mov ax,[ds:bx][$0b]; { number of first data sector }
@ende:
 pop ds;
end;


function totalSectors (Disk : char) : longint; assembler;
asm
 mov al,Disk; push ax; call totalDataSectors;
 push ax; push dx;
 mov al,Disk; push ax; call firstDataSector;
 pop dx; pop bx;
 add ax,bx; adc dx,0;
end;


type
 tControlBlock = record
  start  : longint;
  number : word;
  bufptr : pointer;
 end;

var ControlBlock : tControlblock;

function SectorRead (Drive : char; first : longint; anz : word; var buffer) : boolean; assembler;
asm
 push ds; push bp;
 mov bx,SEG(ControlBlock); mov ds,bx; mov bx,OFFSET(ControlBlock);
 lea di,first;
 mov ax,ss:[di][0]; mov ds:[bx][0],ax;  { first Lo-Word -> Start Lo-Word }
 mov ax,ss:[di][2]; mov ds:[bx][2],ax;  { first Hi-Word -> Start Hi-Word }
 mov ax,ss:[di][-2]; mov ds:[bx][4],ax; { anz -> number }
 mov ax,ss:[di][-6]; mov ds:[bx][6],ax; { @buffer Lo-Word -> bufptr Lo-Word }
 mov ax,ss:[di][-4]; mov ds:[bx][8],ax; { @buffer Hi-Word -> bufptr Hi-Word }
 mov al,Drive; or al,$20; sub al,'a'; { Laufwerksnummer ermitteln, hier 0:A 1:B usw. }
 mov cx,-1;
 int $25; pop bx; { Dos - absolute read aufrufen }
 { CF gesetzt, wenn Fehler }
 pop bp; pop ds;
 cmc; mov al,0; adc al,al; { false(=0) -> Fehler; true(=1) -> kein Fehler }
end;

function SectorWrite (Drive : char; first : longint; anz : word; var buffer) : boolean; assembler;
asm
 push ds; push bp;
 mov bx,SEG(ControlBlock); mov ds,bx; mov bx,OFFSET(ControlBlock);
 lea di,first;
 mov ax,ss:[di][0]; mov ds:[bx][0],ax;  { first Lo-Word -> Start Lo-Word }
 mov ax,ss:[di][2]; mov ds:[bx][2],ax;  { first Hi-Word -> Start Hi-Word }
 mov ax,ss:[di][-2]; mov ds:[bx][4],ax; { anz -> number }
 mov ax,ss:[di][-6]; mov ds:[bx][6],ax; { @buffer Lo-Word -> bufptr Lo-Word }
 mov ax,ss:[di][-4]; mov ds:[bx][8],ax; { @buffer Hi-Word -> bufptr Hi-Word }
 mov al,Drive; or al,$20; sub al,'a'; { Laufwerksnummer ermitteln, hier 0:A 1:B usw. }
 mov cx,-1;
 int $26; pop bx; { Dos - absolute write aufrufen }
 { CF gesetzt, wenn Fehler }
 pop bp; pop ds;
 cmc; mov al,0; adc al,al; { false(=0) -> Fehler; true(=1) -> kein Fehler }
end;


end.
